SystemOrganization addCategory: #'PolySource-Core'!
SystemOrganization addCategory: #'PolySource-Model'!
SystemOrganization addCategory: #'PolySource-Morphic'!
SystemOrganization addCategory: #'PolySource-Network'!
SystemOrganization addCategory: #'PolySource-Tests'!

WorldState subclass: #PSRemoteWorldState
	instanceVariableNames: 'deferredUIMessages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PolySource-Morphic'!

!PSRemoteWorldState commentStamp: 'mt 9/7/2010 12:14:23.771' prior: 0!
I have my own queue with deferred UI messages. My dynamic scope is changed to the remote client of the corresponding remote world in EVERY update cycle.!

!PSRemoteWorldState methodsFor: 'synchronization' stamp: 'mt 9/7/2010 11:59:35.716'!
addDeferredUIMessage: valuableObject

	self deferredUIMessages nextPut: valuableObject.! !

!PSRemoteWorldState methodsFor: 'synchronization' stamp: 'mt 9/7/2010 11:59:48.829'!
deferredUIMessages

	^ deferredUIMessages ifNil: [deferredUIMessages := SharedQueue new]
! !

!PSRemoteWorldState methodsFor: 'update cycle' stamp: 'mt 9/7/2010 12:12:23.233'!
doOneCycleFor: aRemoteWorld

	PSCurrentRemoteClient
		value: aRemoteWorld remoteClient
		during: [super doOneCycleFor: aRemoteWorld].! !

!PSRemoteWorldState methodsFor: 'stepping' stamp: 'mt 9/7/2010 12:01:24.095'!
runStepMethodsIn: aWorld
	"Perform periodic activity inbetween event cycles"
	| queue msg limit stamp |
	"Limit processing of deferredUIMessages to a max. amount of time"
	limit := self class deferredExecutionTimeLimit.
	stamp := Time millisecondClockValue.
	queue := self deferredUIMessages.
	[(Time millisecondsSince: stamp) >= limit 
		or:[(msg := queue nextOrNil) == nil]] 
			whileFalse: [msg value].
	self runLocalStepMethodsIn: aWorld.! !

SqEntity subclass: #PSSourceCode
	instanceVariableNames: 'uuid snapshot updates categories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PolySource-Model'!

!PSSourceCode methodsFor: 'accessing' stamp: 'mt 8/31/2010 10:49:43.319'!
addCategory: aSymbol

	self categories add: aSymbol.
	dummy := #categoryAdded.
	self categoryAdded: aSymbol.! !

!PSSourceCode methodsFor: 'accessing' stamp: 'mt 8/30/2010 13:31:34.438'!
addUpdate: aCodeUpdate

	self updates add: aCodeUpdate.
	dummy := #snapshotUpdated.
	self snapshotUpdated: aCodeUpdate.! !

!PSSourceCode methodsFor: 'accessing' stamp: 'mt 9/7/2010 14:40:04.116'!
categories

	^ categories! !

!PSSourceCode methodsFor: 'signals' stamp: 'mt 9/7/2010 14:31:02.177'!
categoryAdded: aSymbol

	self emitAlways.! !

!PSSourceCode methodsFor: 'updating' stamp: 'mt 8/31/2010 11:02:48.477'!
fileInSnapshot
	"Load the source of all categories in the current snapshot into the image."
	
	snapshot keysAndValuesDo: [:category :source | | strm |
		(ReadStream on: source) fileIn].! !

!PSSourceCode methodsFor: 'initialize' stamp: 'mt 9/7/2010 14:40:16.498'!
initialize

	super initialize.
	categories := OrderedCollection new.
	updates := OrderedCollection new.! !

!PSSourceCode methodsFor: 'accessing' stamp: 'mt 8/30/2010 13:26:57.291'!
snapshot

	^ snapshot! !

!PSSourceCode methodsFor: 'accessing' stamp: 'mt 9/8/2010 09:01:14.756'!
snapshot: aCodeSnapshot

	snapshot := aCodeSnapshot.
	updates := OrderedCollection new.
	self snapshotChanged.! !

!PSSourceCode methodsFor: 'signals' stamp: 'mt 9/7/2010 14:31:09.193'!
snapshotChanged

	self emitAlways.! !

!PSSourceCode methodsFor: 'signals' stamp: 'mt 9/7/2010 14:31:13.765'!
snapshotUpdated: anUpdate

	self emitAlways.! !

!PSSourceCode methodsFor: 'updating' stamp: 'mt 8/31/2010 11:02:25.772'!
updateSnapshot
	"Extract the source code of all registered categories into a dictionary."
	
	| updatedSnapshot |
	updatedSnapshot := Dictionary new.
	
	self categories do: [:cat | | strm |
		strm := WriteStream on: (String new: 100).
		SystemOrganization fileOutCategory: cat on: strm.
		updatedSnapshot at: cat put: strm contents].
	
	self snapshot: updatedSnapshot.
	self snapshotChanged.! !

!PSSourceCode methodsFor: 'accessing' stamp: 'mt 9/7/2010 14:40:19.878'!
updates

	^ updates! !

!PSSourceCode methodsFor: 'accessing' stamp: 'mt 8/30/2010 15:26:59.314'!
uuid

	^ uuid! !

!PSSourceCode methodsFor: 'accessing' stamp: 'mt 8/30/2010 15:27:09.666'!
uuid: aUUID

	uuid := aUUID.! !

SqEntity subclass: #PSSourceCodeRepository
	instanceVariableNames: 'sourceCodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PolySource-Model'!

!PSSourceCodeRepository methodsFor: 'add/remove' stamp: 'mt 9/6/2010 11:23:39.632'!
addSourceCode: aSourceCode

	self sourceCodes add: aSourceCode.
	dummy := #sourceCodeAdded.
	^ aSourceCode! !

!PSSourceCodeRepository methodsFor: 'initialize-release' stamp: 'mt 9/7/2010 14:39:21.48'!
initialize

	super initialize.
	sourceCodes := OrderedCollection new.! !

!PSSourceCodeRepository methodsFor: 'add/remove' stamp: 'mt 9/6/2010 11:24:56.698'!
removeSourceCode: aSourceCode

	self sourceCode remove: aSourceCode ifAbsent: [].
	dummy := #sourceCodeRemoved.
	^ aSourceCode! !

!PSSourceCodeRepository methodsFor: 'accessing' stamp: 'mt 9/6/2010 11:24:32.092'!
sourceCodeByID: aUUID

	^ self sourceCodes
		detect: [:c | c uuid = aUUID]
		ifNone: [self addSourceCode: (PSSourceCode new uuid: aUUID)]! !

!PSSourceCodeRepository methodsFor: 'accessing' stamp: 'mt 9/7/2010 14:39:10.491'!
sourceCodes

	^ sourceCodes! !

SqEntity subclass: #PSSourceCodeUpdate
	instanceVariableNames: 'event'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PolySource-Model'!

!PSSourceCodeUpdate methodsFor: 'accessing' stamp: 'mt 9/8/2010 08:48:39.234'!
event
	
	^ event! !

!PSSourceCodeUpdate methodsFor: 'accessing' stamp: 'mt 9/7/2010 15:52:39.128'!
event: aSystemEvent
	
	event := self transformEvent: aSystemEvent.! !

!PSSourceCodeUpdate methodsFor: 'accessing' stamp: 'mt 9/7/2010 19:48:34.865'!
eventItemClass

	^ (self event instVarNamed: #environment) at: AbstractEvent classKind! !

!PSSourceCodeUpdate methodsFor: 'accessing' stamp: 'mt 9/7/2010 19:48:44.886'!
eventItemMethod

	^ (self event instVarNamed: #environment) at: AbstractEvent methodKind! !

!PSSourceCodeUpdate methodsFor: 'transforming' stamp: 'mt 9/7/2010 17:10:37.353'!
fileOutSourceFor: aCompiledMethod
	"Better than just #getSource. Preserves Protocols."
	
	| strm |
	strm := WriteStream on: (String new: 1000).

	strm header; timeStamp.
	aCompiledMethod methodClass
		printMethodChunk: aCompiledMethod selector
		withPreamble: true
		on: strm
		moveSource: false
		toFile: 0.
		
	^ strm contents! !

!PSSourceCodeUpdate methodsFor: 'transforming' stamp: 'mt 9/8/2010 08:40:33.737'!
transformEvent: anEvent

	| result env |
	result := anEvent copy.
	env := (anEvent instVarNamed: #environment) copy.

	"Use custom environment."
	result instVarNamed: #environment put: env.

	"Generic transformations."
	result itemClass ifNotNil: [result itemClass: result itemClass name asSymbol].
	result itemRequestor ifNotNil: [result itemRequestor: nil]. "Cannot transport requestor instance anyway."
	result itemMethod ifNotNil: [result itemMethod: result itemMethod selector].

	"Kind-Specific transformations."
	anEvent itemKind = AbstractEvent classKind ifTrue: [
		result instVarNamed: #item put: anEvent item name asSymbol. "class name by default"
		(anEvent isAdded or: [anEvent isModified])
			ifTrue: [result instVarNamed: #item put: anEvent item definition]. "class definition"
		anEvent isModified ifTrue: [result oldItem: anEvent oldItem name asSymbol]. "old class name"
		anEvent isCommented ifTrue: [result instVarNamed: #item put: anEvent item comment]. "class comment"
		anEvent isReorganized ifTrue: [
			result instVarNamed: #item put: {
				(anEvent item organization instVarNamed: #categoryArray) copy.
				(anEvent item organization instVarNamed: #categoryStops) copy.
				(anEvent item organization instVarNamed: #elementArray) copy}]. "class protocols"].
		
	anEvent itemKind = AbstractEvent methodKind ifTrue: [
		result instVarNamed: #item put: anEvent item selector. "method selector by default"
		(anEvent isAdded or: [anEvent isModified]) ifTrue: [
			result instVarNamed: #item put: (self fileOutSourceFor: anEvent item)]. "method source" 
		anEvent isModified ifTrue: [result oldItem: anEvent oldItem selector]. "old selector"].
	
	^ result! !

Dictionary subclass: #PSFixtures
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PolySource-Tests'!

PSFixtures class
	instanceVariableNames: 'instance'!

PSFixtures class
	instanceVariableNames: 'instance'!

!PSFixtures class methodsFor: 'accessing' stamp: 'mig 8/9/2010 17:30:19.665'!
at: key

	^ self instance at: key! !

!PSFixtures class methodsFor: 'accessing' stamp: 'mig 8/9/2010 17:30:01.619'!
at: key put: value
.
	^ self instance at: key put: value! !

!PSFixtures class methodsFor: 'examples' stamp: 'mt 9/6/2010 11:46:25.931'!
example01

	"Need a proxy to get the class compiled into the right dictionary."
	PSSystemDictionaryProxy install.

	PSCurrentRemoteClient
		value: (PSFixtures at: #example01 put: PSRemoteClient new)
		during: [Object
			subclass: #PSFoobar
			instanceVariableNames: ''
			classVariableNames: ''
			poolDictionaries: ''
			category: 'PolySource-Tests-Temporary'].

	"Test data prepared. Clean up."
	Smalltalk globals uninstall.! !

!PSFixtures class methodsFor: 'testing' stamp: 'mig 8/25/2010 13:53:39.589'!
includesKey: key

	^ self instance includesKey: key! !

!PSFixtures class methodsFor: 'singleton' stamp: 'mig 8/9/2010 17:29:05.257'!
instance

	^ instance ifNil: [instance := self new]! !

DynamicVariable subclass: #PSCurrentRemoteClient
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PolySource-Core'!

PasteUpMorph subclass: #PSRemoteWorld
	instanceVariableNames: 'remoteClient infoLabel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PolySource-Morphic'!

!PSRemoteWorld commentStamp: 'mt 9/7/2010 12:15:36.029' prior: 0!
A remote world has a remote world state and a remote client. =D!

!PSRemoteWorld class methodsFor: 'project' stamp: 'mt 9/7/2010 12:03:27.714'!
newWorldForProject: projectOrNil 

	^ self new initForProject: PSRemoteWorldState new! !

!PSRemoteWorld methodsFor: 'initialize' stamp: 'mt 9/7/2010 13:12:04.365'!
initialize

	super initialize.
	self
		color: (Color gray: 0.8);
		borderWidth: 0.
		
	infoLabel := TextMorph new lock.
	self addMorphBack: infoLabel.
	infoLabel topLeft: self topLeft + (20@50).
	
	self connect: self signal: #remoteClient: toSelector: #updateInfoLabel.
	self updateInfoLabel.! !

!PSRemoteWorld methodsFor: 'accessing' stamp: 'mt 9/6/2010 11:26:10.299'!
remoteClient

	^ remoteClient! !

!PSRemoteWorld methodsFor: 'accessing' stamp: 'mt 9/7/2010 12:18:58.166'!
remoteClient: aRemoteClient

	remoteClient := aRemoteClient.! !

!PSRemoteWorld methodsFor: 'initialize' stamp: 'mt 9/7/2010 13:13:07.697'!
updateInfoLabel

	infoLabel contents: 
		(self remoteClient
			ifNil: ['<font size="12" color="#FF0000"><b>No remote client configured.</b></font>' asHtmlText]
			ifNotNil: ['']).! !

!PSRemoteWorld methodsFor: 'accessing' stamp: 'mt 9/7/2010 13:15:56.106'!
worldState
	"Needed to grant access to deferred UI message queue. See PSRemoteWorldState>>deferredUIMessages."
	
	^ worldState! !

!SystemOrganizer methodsFor: '*polysource-core' stamp: 'mt 8/30/2010 11:30:34.255'!
isProxy

	^ false! !

SystemDictionary subclass: #PSSystemDictionary
	instanceVariableNames: 'organization'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PolySource-Core'!

!PSSystemDictionary commentStamp: 'mt 8/29/2010 19:23:48.53' prior: 0!
I am a copy of the local SystemDictionary (Smalltalk globals) but I do not copy the classes. I provide my own organization.!

!PSSystemDictionary methodsFor: 'initialize' stamp: 'mig 8/9/2010 15:00:32.549'!
initialize: size

	super initialize: size.
	Smalltalk globals keysAndValuesDo: [ :key :value |
		self at: key put: value].! !

!SystemDictionary methodsFor: '*polysource-core' stamp: 'mt 8/29/2010 19:32:52.227'!
isProxy

	^ false! !

TestCase subclass: #PSBasicTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PolySource-Tests'!

!PSBasicTestCase methodsFor: 'tests' stamp: 'mig 8/9/2010 15:22:47.123'!
test01new

	| sut |
	sut := PSSystemDictionary new.

	self assert: sut size equals: Smalltalk globals size.
	Smalltalk globals keysAndValuesDo: [:key :value |
		self assert: value == (sut at: key)].
! !

!PSBasicTestCase methodsFor: 'tests' stamp: 'mig 8/10/2010 13:59:49.758'!
test02proxy

	| testCase sut root |
	
	sut := PSSystemDictionaryProxy new.
	root := Smalltalk globals.
	
	testCase := Smalltalk globals at: #TestCase.
	
	sut install.
	self assert: sut == Smalltalk globals.
	self assert: testCase == (Smalltalk globals at: #TestCase).
	sut uninstall.
	self assert: root == Smalltalk globals.! !

!PSBasicTestCase methodsFor: 'tests' stamp: 'mt 8/30/2010 11:55:22.347'!
test03proxyWithOrganization

	PSSystemDictionaryProxy install.
	
	self
		assert: Smalltalk globals isProxy;
		assert: SystemOrganization isProxy.
		
	PSSystemDictionaryProxy uninstall.
	
	self
		deny: Smalltalk globals isProxy;
		deny: SystemOrganization isProxy.! !

TestCase subclass: #PSModelTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PolySource-Tests'!

!PSModelTestCase methodsFor: 'tests' stamp: 'mt 9/6/2010 11:21:18.835'!
test01snapshots

	| cc |
	cc := PSSourceCode new.
	cc
		addCategory: #'PolySource-Core';
		addCategory: #'PolySource-Model'.
		
	self assert: cc snapshot isNil.
	
	cc updateSnapshot.
	
	self
		assert: 2 equals: cc snapshot size;
		assert: (cc snapshot includesKey: #'PolySource-Core');
		assert: (cc snapshot includesKey: #'PolySource-Model');
		assert: (cc snapshot associations first value isString).! !

!PSModelTestCase methodsFor: 'tests' stamp: 'mt 9/7/2010 19:49:59.483'!
test02classAddedEvent

	| evt upd |
	upd := PSSourceCodeUpdate new.
	evt := AddedEvent item: String kind: #class.
	upd event: evt.
	
	self
		assert: String name equals: upd eventItemClass;
		assert: String definition equals: upd event item.
	! !

!PSModelTestCase methodsFor: 'tests' stamp: 'mt 9/7/2010 19:55:36.755'!
test03classModifiedEvent

	| evt upd |
	upd := PSSourceCodeUpdate new.
	evt := ModifiedClassDefinitionEvent classDefinitionChangedFrom: String to: Integer.
	upd event: evt.
	
	self
		assert: Integer name equals: upd eventItemClass;
		assert: Integer definition equals: upd event item;
		assert: String name equals: upd event oldItem.
	! !

!PSModelTestCase methodsFor: 'tests' stamp: 'mt 9/7/2010 19:57:31.449'!
test04classRemovedEvent

	| evt upd |
	upd := PSSourceCodeUpdate new.
	evt := RemovedEvent item: String kind: #class.
	upd event: evt.
	
	self
		assert: String name equals: upd eventItemClass;
		assert: String name equals: upd event item.
	! !

!PSModelTestCase methodsFor: 'tests' stamp: 'mt 9/7/2010 20:01:36.303'!
test05classRenamedEvent

	| evt upd |
	upd := PSSourceCodeUpdate new.
	evt := RenamedEvent class: String category: #'Collections-Strings' oldName: #String newName: #NewString.
	upd event: evt.
	
	self
		assert: String name equals: upd eventItemClass;
		assert: String name equals: upd event item.
	! !

!PSModelTestCase methodsFor: 'tests' stamp: 'mt 9/7/2010 20:02:51.206'!
test06classCommentedEvent

	| evt upd |
	upd := PSSourceCodeUpdate new.
	evt := CommentedEvent item: String kind: #class.
	upd event: evt.
	
	self
		assert: String name equals: upd eventItemClass;
		assert: String comment equals: upd event item.
	! !

!PSModelTestCase methodsFor: 'tests' stamp: 'mt 9/7/2010 20:05:08.314'!
test07classRecategorizedEvent

	| evt upd |
	upd := PSSourceCodeUpdate new.
	evt := RecategorizedEvent class: String category: #Foobar oldCategory: #'Collections-Strings'.
	upd event: evt.
	
	self
		assert: String name equals: upd eventItemClass;
		assert: String name equals: upd event item.
	! !

!PSModelTestCase methodsFor: 'tests' stamp: 'mt 9/7/2010 20:14:27.054'!
test08classReorganizedEvent

	| evt upd |
	upd := PSSourceCodeUpdate new.
	evt := ReorganizedEvent item: String kind: #class.
	upd event: evt.
	
	self
		assert: String name equals: upd eventItemClass;
		assert: String organization elementArray equals: upd event item third;
		assert: String organization categories equals: upd event item first.
	! !

!PSModelTestCase methodsFor: 'tests' stamp: 'mt 9/8/2010 08:41:29.066'!
test09methodAddedEvent

	| evt upd |
	upd := PSSourceCodeUpdate new.
	evt := AddedEvent item: String>>#stringhash kind: #method.
	evt itemClass: String.
	upd event: evt.
	
	self
		assert: String name equals: upd eventItemClass;
		assert: #stringhash equals: upd eventItemMethod;
		assert: (upd event item includesSubString: evt item getSource).
	! !

!PSModelTestCase methodsFor: 'tests' stamp: 'mig 9/8/2010 11:04:34.931'!
test10methodModifiedEvent

	| evt upd |
	upd := PSSourceCodeUpdate new.
	evt := ModifiedEvent methodChangedFrom: String>>#stringhash to: String>>#withCRs selector: #withCRs inClass: String.
	upd event: evt.
	
	self
		assert: String name equals: upd eventItemClass;
		assert: #withCRs equals: upd eventItemMethod;
		assert: #stringhash equals: upd event oldItem;
		assert: (upd event item includesSubString: evt item getSource).
	! !

!PSModelTestCase methodsFor: 'tests' stamp: 'mig 9/8/2010 11:04:52.165'!
test11methodRecategorizedEvent

	| evt upd |
	upd := PSSourceCodeUpdate new.
	evt := RecategorizedEvent method: String>>#withCRs protocol: #testing class: String oldProtocol: #printing. 
	upd event: evt.
	
	self
		assert: String name equals: upd eventItemClass;
		assert: #withCRs equals: upd eventItemMethod;
		assert: #testing equals: upd event itemProtocol;
		assert: #printing equals: upd event oldCategory.
	! !

!PSModelTestCase methodsFor: 'tests' stamp: 'mt 9/8/2010 08:47:43.812'!
test12methodRemovedEvent

	| evt upd |
	upd := PSSourceCodeUpdate new.
	evt := RemovedEvent item: String>>#stringhash kind: #method.
	evt itemClass: String.
	upd event: evt.
	
	self
		assert: String name equals: upd eventItemClass;
		assert: #stringhash equals: upd eventItemMethod;
		assert: #stringhash equals: upd event item.
	! !

TestCase subclass: #PSNetworkTestCase
	instanceVariableNames: 'srv clt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PolySource-Tests'!

!PSNetworkTestCase methodsFor: 'running' stamp: 'mt 9/6/2010 11:10:43.548'!
setUp

	super setUp.
	
	srv := SyServer new start.
	clt := PSLocalClient new
		connection: (SyClient new connectToHostNamed: 'localhost' port: 22701).! !

!PSNetworkTestCase methodsFor: 'running' stamp: 'mt 8/30/2010 15:54:46.481'!
tearDown

	super tearDown.

	clt connection disconnect.

	srv stop.
	srv repository clear.
! !

!PSNetworkTestCase methodsFor: 'tests' stamp: 'mig 9/8/2010 11:24:33.903'!
test01uuid

	| src |

	self
		assert: (src := clt sourceCode) notNil;
		assert: clt uuid equals: src uuid.
	
	
! !

TestCase subclass: #PSProxiedTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PolySource-Tests'!

!PSProxiedTestCase methodsFor: 'running' stamp: 'mt 8/29/2010 19:41:49.115'!
setUp

	super setUp.
	PSSystemDictionaryProxy install.! !

!PSProxiedTestCase methodsFor: 'running' stamp: 'mig 8/10/2010 14:00:41.469'!
tearDown

	Smalltalk globals uninstall.
	super tearDown.
! !

!PSProxiedTestCase methodsFor: 'tests' stamp: 'mt 9/6/2010 11:37:49.751'!
test01environment

	| newRemoteClient |
	newRemoteClient := PSRemoteClient new.
	
	PSCurrentRemoteClient value: newRemoteClient during: [
		self assert: Object environment yourself == newRemoteClient systemDictionary].
	self deny: Object environment yourself == newRemoteClient systemDictionary.
! !

!PSProxiedTestCase methodsFor: 'tests' stamp: 'mt 9/6/2010 11:37:08.187'!
test02newClasses

	self deny: (Smalltalk includesKey: #PSFoobar).
	self deny: (Smalltalk includesKey: #PSFoobar2).
	
	PSCurrentRemoteClient value: PSRemoteClient new during: [
		"Create a class and a subclass of it."
		Object subclass: #PSFoobar instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: 'PolySource-Tests-Temporary'.
		(Smalltalk at: #PSFoobar)
			subclass: #PSFoobar2 instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: 'PolySource-Tests-Temporary'.
		self assert: (Smalltalk includesKey: #PSFoobar).
		self assert: (Smalltalk includesKey: #PSFoobar2)].
	
	self deny: (Smalltalk includesKey: #PSFoobar).
	self deny: (Smalltalk includesKey: #PSFoobar2).
! !

!PSProxiedTestCase methodsFor: 'tests' stamp: 'mt 9/6/2010 11:36:53.926'!
test03evaluate

	self deny: (Smalltalk includesKey: #PSFoobar).
	
	PSCurrentRemoteClient value: PSRemoteClient new during: [
		"Create a class and instantiate it once."
		Object subclass: #PSFoobar instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: 'PolySource-Tests-Temporary'.
		self
			shouldnt: [Compiler evaluate: 'PSFoobar new']
			raise: MessageNotUnderstood].
		
	self
		should: [Compiler evaluate: 'PSFoobar new']
		raise: MessageNotUnderstood.
! !

!PSProxiedTestCase methodsFor: 'tests' stamp: 'mt 8/29/2010 19:56:47.815'!
test05proxyTest

	PSSystemDictionaryProxy uninstall.
	self deny: Smalltalk globals isProxy.

	PSSystemDictionaryProxy install.
	self assert: Smalltalk globals isProxy.! !

!PSProxiedTestCase methodsFor: 'tests' stamp: 'mig 9/7/2010 09:57:21.031'!
test06organization

	PSCurrentRemoteClient value: PSRemoteClient new during: [
		| foobar foobar2 |
		Object subclass: #PSFoobar instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: 'PolySource-Tests-Temporary'.
		foobar := Compiler evaluate: 'PSFoobar'.
		foobar subclass: #PSFoobar2 instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: 'PolySource-Tests-Temporary'.
		foobar2 := Compiler evaluate: 'PSFoobar2'.
		self assert: #'PolySource-Tests-Temporary' equals: foobar category.
		self assert: #'PolySource-Tests-Temporary' equals: foobar2 category.
	].! !

Object subclass: #PSLocalClient
	instanceVariableNames: 'uuid connection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PolySource-Network'!

PSLocalClient class
	instanceVariableNames: 'instance'!

!PSLocalClient commentStamp: 'mt 9/7/2010 16:11:14.467' prior: 0!
Hallo.!

PSLocalClient class
	instanceVariableNames: 'instance'!

!PSLocalClient class methodsFor: 'instance' stamp: 'mt 9/7/2010 10:06:47.765'!
instance
	"This is the instance of PSLocalClient that has to be (re-)used in a normal application. Tests are allowed to create further instances with own SqSync connections and UUIDs."
	
	^ instance ifNil: [instance := self new]! !

!PSLocalClient class methodsFor: 'instance' stamp: 'mt 9/7/2010 10:07:11.685'!
reset
	
	instance := nil.! !

!PSLocalClient methodsFor: 'networking' stamp: 'mt 9/7/2010 13:55:29.778'!
connectTo: aHostnameOrAddressString

	self disconnect.
		
	self connection: (SyClient new
		connectToHostNamed: aHostnameOrAddressString
		port: 22701).! !

!PSLocalClient methodsFor: 'accessing' stamp: 'mt 8/30/2010 15:34:56.859'!
connection

	^ connection! !

!PSLocalClient methodsFor: 'accessing' stamp: 'mt 9/7/2010 13:57:20.092'!
connection: anSyClient

	connection := anSyClient.
	
	connection ifNil: [^ self].
	
	"Assure the root object resp. source code repository."
	connection repository rootObject
		ifNil: [connection repository rootObject: PSSourceCodeRepository new]! !

!PSLocalClient methodsFor: 'notifications' stamp: 'mt 9/7/2010 15:30:26.088'!
disableNotificationProcessing

	SystemChangeNotifier uniqueInstance
		noMoreNotificationsFor: self.! !

!PSLocalClient methodsFor: 'networking' stamp: 'mt 9/7/2010 14:18:43.207'!
disconnect

	connection ifNil: [^ self].
	connection disconnect.
	connection := nil.! !

!PSLocalClient methodsFor: 'notifications' stamp: 'mt 9/7/2010 15:30:53.685'!
enableNotificationProcessing

	SystemChangeNotifier uniqueInstance
		notify: self ofAllSystemChangesUsing: #processNotification:.! !

!PSLocalClient methodsFor: 'notifications' stamp: 'mt 9/7/2010 17:35:20.214'!
processNotification: anEvent

	| sourceCode |
	anEvent isDoIt ifTrue: [^ self].
	
	Transcript cr; show: anEvent itemKind asString, ' ', anEvent changeKind asString, ' ', anEvent itemCategory.
	
	sourceCode := (PSCurrentRemoteClient value ifNil: [self]) sourceCode.
	sourceCode ifNil: [^ self].
	
	anEvent itemKind = AbstractEvent categoryKind ifTrue: [
		anEvent isAdded ifTrue: [^ self].
		anEvent isModified ifTrue: [^ self].
		(anEvent isRenamed and: [sourceCode categories includes: anEvent oldName])
			ifFalse: [^ self].
		(anEvent isRemoved and: [sourceCode categories includes: anEvent itemCategory])
			ifFalse: [^ self]].
	
	(sourceCode categories includes: anEvent itemClass category)
		ifFalse: [^ self].
		
	sourceCode addUpdate: (PSSourceCodeUpdate new event: anEvent).! !

!PSLocalClient methodsFor: 'accessing' stamp: 'mig 9/7/2010 15:05:50.038'!
remoteClients

	^ self connection repository rootObject sourceCodes collect: [:sc |
		PSRemoteClient new sourceCode: sc]! !

!PSLocalClient methodsFor: 'accessing' stamp: 'mt 9/6/2010 11:25:19.911'!
sourceCode

	self connection ifNil: [^ nil].
	self connection repository rootObject ifNil: [^ nil].
	
	^ self connection repository rootObject sourceCodeByID: self uuid! !

!PSLocalClient methodsFor: 'accessing' stamp: 'mt 8/30/2010 15:24:04.959'!
uuid

	^ uuid ifNil: [uuid := UUID new]! !

Object subclass: #PSRemoteClient
	instanceVariableNames: 'systemDictionary systemOrganization sourceCode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PolySource-Network'!

!PSRemoteClient class methodsFor: 'instance creation' stamp: 'mt 9/6/2010 11:17:32.085'!
new

	PSCurrentRemoteClient value ifNotNil: [self signal: 'You have to reset the scope to create a new remote client object!!'].
	^ super new! !

!PSRemoteClient methodsFor: 'accessing' stamp: 'mt 9/6/2010 11:18:05.938'!
initialize

	super initialize.
	
	systemDictionary := PSSystemDictionary new.
	systemOrganization := SystemOrganization deepCopy.! !

!PSRemoteClient methodsFor: 'accessing' stamp: 'mt 9/7/2010 13:11:20.034'!
name

	^ self sourceCode
		ifNil: [super name]
		ifNotNil: [self sourceCode uuid asString]! !

!PSRemoteClient methodsFor: 'accessing' stamp: 'mt 9/6/2010 11:34:12.155'!
sourceCode

	^ sourceCode! !

!PSRemoteClient methodsFor: 'accessing' stamp: 'mt 9/6/2010 11:34:20.648'!
sourceCode: aSourceCode

	sourceCode := aSourceCode.! !

!PSRemoteClient methodsFor: 'accessing' stamp: 'mt 9/6/2010 11:18:13.62'!
systemDictionary

	^ systemDictionary! !

!PSRemoteClient methodsFor: 'accessing' stamp: 'mt 9/6/2010 11:18:10.182'!
systemOrganization

	^ systemOrganization! !

MorphicProject subclass: #PSRemoteProject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PolySource-Morphic'!

!PSRemoteProject commentStamp: 'mt 9/7/2010 12:15:12.813' prior: 0!
A remote projecte has a remote world. ;)!

!PSRemoteProject methodsFor: 'initialize' stamp: 'mt 9/7/2010 10:57:24.923'!
initialize

	self perform: #initialize withArguments: #() inSuperclass: Project.

	world := PSRemoteWorld newWorldForProject: self.
	Locale switchToID: CurrentProject localeID.
	Preferences useVectorVocabulary ifTrue: [world installVectorVocabulary]
	! !

!PSRemoteProject methodsFor: 'accessing' stamp: 'mt 9/7/2010 13:09:47.222'!
name

	^ self world remoteClient
		ifNil: ['nil']
		ifNotNil: [self world remoteClient name]! !

!Class methodsFor: '*polysource-core-override' stamp: 'mig 8/9/2010 16:23:00.596'!
environment

	^ Smalltalk globals
	"environment == nil ifTrue: [^ super environment].
	^ environment"! !

ProtoObject subclass: #PSSystemDictionaryProxy
	instanceVariableNames: 'original'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PolySource-Core'!

!PSSystemDictionaryProxy class methodsFor: 'installation' stamp: 'mig 8/9/2010 17:42:02.667'!
install

	self new install.! !

!PSSystemDictionaryProxy class methodsFor: 'installation' stamp: 'mig 8/9/2010 17:42:35.209'!
uninstall

	Smalltalk globals uninstall.! !

!PSSystemDictionaryProxy methodsFor: 'proxy' stamp: 'mt 9/6/2010 11:44:05.53'!
doesNotUnderstand: aMessage

	original ifNil: [^ super doesNotUnderstand: aMessage].
	
	^ aMessage sendTo: (PSCurrentRemoteClient value
		ifNil: [original]
		ifNotNil: [PSCurrentRemoteClient value systemDictionary])! !

!PSSystemDictionaryProxy methodsFor: 'installation' stamp: 'mt 8/30/2010 11:49:01.349'!
install

	self isInstalled ifTrue: [^self].

	original := Smalltalk globals.
	Smalltalk instVarNamed: #globals put: self.
	
	"Add a proxy for the organization too."
	PSSystemOrganizerProxy install.
! !

!PSSystemDictionaryProxy methodsFor: 'testing' stamp: 'mig 8/9/2010 15:14:19.08'!
isInstalled

	^ Smalltalk globals == self! !

!PSSystemDictionaryProxy methodsFor: 'testing' stamp: 'mt 8/29/2010 19:32:19.736'!
isProxy

	^ true! !

!PSSystemDictionaryProxy methodsFor: 'installation' stamp: 'mt 8/30/2010 11:49:48.948'!
uninstall

	self isInstalled ifFalse: [^self].

	Smalltalk instVarNamed: #globals put: original.
	original := nil.
	
	"Remove proxy for organization."
	PSSystemOrganizerProxy uninstall.! !

ProtoObject subclass: #PSSystemOrganizerProxy
	instanceVariableNames: 'original'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PolySource-Core'!

!PSSystemOrganizerProxy commentStamp: 'mt 8/30/2010 11:52:21.467' prior: 0!
DO NOT install me directly but use "PSSystemDictionaryProxy install" instead.!

!PSSystemOrganizerProxy class methodsFor: 'as yet unclassified' stamp: 'mt 8/30/2010 11:37:54.726'!
install

	self new install.! !

!PSSystemOrganizerProxy class methodsFor: 'as yet unclassified' stamp: 'mt 8/30/2010 11:38:13.074'!
uninstall

	SystemOrganization uninstall.! !

!PSSystemOrganizerProxy methodsFor: 'as yet unclassified' stamp: 'mt 9/6/2010 11:43:40.372'!
doesNotUnderstand: aMessage

	original ifNil: [^ super doesNotUnderstand: aMessage].
	
	^ aMessage sendTo: (PSCurrentRemoteClient value 
		ifNil: [original]
		ifNotNil: [PSCurrentRemoteClient value systemOrganization])! !

!PSSystemOrganizerProxy methodsFor: 'as yet unclassified' stamp: 'mt 8/30/2010 11:37:01.302'!
install

	self isInstalled ifTrue: [^self].

	original := SystemOrganization.
	Smalltalk at: #SystemOrganization put: self.
! !

!PSSystemOrganizerProxy methodsFor: 'as yet unclassified' stamp: 'mt 8/30/2010 11:37:27.065'!
isInstalled

	^ SystemOrganization == self! !

!PSSystemOrganizerProxy methodsFor: 'as yet unclassified' stamp: 'mt 8/30/2010 11:29:56.853'!
isProxy

	^ true! !

!PSSystemOrganizerProxy methodsFor: 'as yet unclassified' stamp: 'mt 8/30/2010 11:37:15.148'!
uninstall

	self isInstalled ifFalse: [^self].
	
	Smalltalk at: #SystemOrganization put: original.
	original := nil.! !
